@@ -1443,6 +1443,23 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
       h->value.data = (u_char*)link;
     }
 
+    // Add Location header
+    char const* location = headers_get_location(headers);
+    if(location && location[0])
+    {
+      ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
+      if(h == NULL)
+      {
+        return NGX_HTTP_INTERNAL_SERVER_ERROR;
+      }
+      h->hash = 1;
+
+      h->key.len = sizeof("Location") - 1;
+      h->key.data = (u_char*)"Location";
+      h->value.len = strlen(location);
+      h->value.data = (u_char*)location;
+    }
+
     // Add the X-USP-Content-ID header for start/next
     char const* content_id = headers_get_content_id(headers);
     if(content_id && content_id[0])
@@ -1460,6 +1477,40 @@ ngx_streaming_handler_get(ngx_http_request_t *r,
       h->value.data = (u_char*)content_id;
     }
 
+    // Add CMSD-Static header
+    char const* cmsd_static = headers_get_cmsd_static(headers);
+    if(cmsd_static)
+    {
+      ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
+      if(h == NULL)
+      {
+        return NGX_HTTP_INTERNAL_SERVER_ERROR;
+      }
+      h->hash = 1;
+
+      h->key.len = sizeof("CMSD-Static") - 1;
+      h->key.data = (u_char*)"CMSD-Static";
+      h->value.len = strlen(cmsd_static);
+      h->value.data = (u_char*)cmsd_static;
+    }
+
+    // Add CMSD-Dynamic header
+    char const* cmsd_dynamic = headers_get_cmsd_dynamic(headers);
+    if(cmsd_dynamic)
+    {
+      ngx_table_elt_t* h = ngx_list_push(&r->headers_out.headers);
+      if(h == NULL)
+      {
+        return NGX_HTTP_INTERNAL_SERVER_ERROR;
+      }
+      h->hash = 1;
+
+      h->key.len = sizeof("CMSD-Dynamic") - 1;
+      h->key.data = (u_char*)"CMSD-Dynamic";
+      h->value.len = strlen(cmsd_dynamic);
+      h->value.data = (u_char*)cmsd_dynamic;
+    }
+
     // Set the content length
     {
       uint64_t content_length = buckets_size(buckets);
